#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "mygrid.hh"

template <class Grid> std::shared_ptr<Grid> constructGrid() {
  Dune::GridFactory<Grid> gridFactory;

  Dune::FieldMatrix<double, 3, DIM> vertices;
  vertices[0] = MyGeometry::A;
  vertices[1] = MyGeometry::B;
  vertices[2] = MyGeometry::C;
  for (size_t i = 0; i < vertices.N(); ++i)
    gridFactory.insertVertex(vertices[i]);

  Dune::GeometryType cell;
  cell.makeTriangle();

  std::vector<std::vector<unsigned int>> simplices = { { 0, 1, 2 } };

  // sanity-check choices of simplices
  for (size_t i = 0; i < simplices.size(); ++i) {
    Dune::FieldMatrix<double, DIM, DIM> check;
    for (size_t j = 0; j < DIM; ++j)
      check[j] = vertices[simplices[i][j + 1]] - vertices[simplices[i][j]];
    assert(check.determinant() > 0);
    gridFactory.insertElement(cell, simplices[i]);
  }
  return std::shared_ptr<Grid>(gridFactory.createGrid());
}

template <class GridView>
MyFaces<GridView>::MyFaces(GridView const &gridView)
    : lower(gridView), right(gridView) {
  using Grid = typename GridView::Grid;
  using LevelGridView = typename Grid::LevelGridView;
  LevelGridView const rootView = gridView.grid().levelView(0);

  lower.insertFacesByProperty([&](
      typename Grid::LeafGridView::Intersection const &in) {
    return isClose(0, in.geometry().center()[1]);
  });

  BoundaryPatch<LevelGridView> upperRoot(rootView);
  upperRoot.insertFacesByProperty([&](
      typename LevelGridView::Intersection const &in) {
    auto const geometry = in.geometry();
    auto const range = boost::irange(0, geometry.corners());
    return boost::algorithm::all_of(range, [&](int i) {
      auto const &co = geometry.corner(i);
      return (isClose(co[1], MyGeometry::A[1]) &&
              isClose(co[0], MyGeometry::A[0])) ||
             (isClose(co[1], MyGeometry::C[1]) &&
              isClose(co[0], MyGeometry::C[0]));
    });
  });
  BoundaryPatchProlongator<Grid>::prolong(upperRoot, upper);

  BoundaryPatch<LevelGridView> rightRoot(rootView);
  rightRoot.insertFacesByProperty([&](
      typename LevelGridView::Intersection const &in) {
    auto const geometry = in.geometry();
    auto const range = boost::irange(0, geometry.corners());
    return boost::algorithm::all_of(range, [&](int i) {
      auto const &co = geometry.corner(i);
      return (isClose(co[1], MyGeometry::C[1]) &&
              isClose(co[0], MyGeometry::C[0])) ||
             (isClose(co[1], MyGeometry::B[1]) &&
              isClose(co[0], MyGeometry::B[0]));
    });
  });
  BoundaryPatchProlongator<Grid>::prolong(rightRoot, right);
}

#include "mygrid_tmpl.cc"
